import os
import pickle
from dataclasses import asdict, dataclass, field

import fire
from dacite import from_dict

from .sources import MiKanProject

data_path = os.path.expanduser("~//.bgmiget")


@dataclass
class Data:
    results: list = field(default_factory=list)
    old_results: list = field(default_factory=list)
    save_path: str = ""


data = Data()
if os.path.exists(data_path):
    data = from_dict(data_class=Data, data=pickle.load(open(data_path, "rb")))


class BgmiGet:
    def __init__(self, source):
        self.source = source()

    def write_results(self):
        data.old_results = data.results
        data.results = self.source.results
        pickle.dump(asdict(data), open(data_path, "wb"))

    def search(self, query):
        self.source.search(query)
        self.write_results()

    def set_save_path(self, path):
        data.save_path = path
        pickle.dump(asdict(data), open(data_path, "wb"))

    def download(self, index="all"):
        self.source.results = data.results
        self.source.download(data.save_path, index)

    def include(self, keyword):
        self.source.results = data.results
        self.source.results = [(text, url)
                               for text, url in self.source.results if keyword in text]
        self.source.show_results()
        self.write_results()

    def exclude(self, keyword):
        self.source.results = data.results
        self.source.results = [(text, url)
                               for text, url in self.source.results if keyword not in text]
        self.source.show_results()
        self.write_results()

    def undo(self):
        self.source.results = data.old_results
        self.source.show_results()
        self.write_results()


def main():

    bgmiget = BgmiGet(source=MiKanProject)
    fire.Fire({
        "search": bgmiget.search,
        "set_save_path": bgmiget.set_save_path,
        "download": bgmiget.download,
        "include": bgmiget.include,
        "exclude": bgmiget.exclude,
        "undo": bgmiget.undo
    })



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bgmiget', 'bgmiget.sources']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'dacite>=1.6.0,<2.0.0',
 'fire>=0.4.0,<0.5.0',
 'requests>=2.28.1,<3.0.0',
 'torrentp>=0.1.5,<0.2.0']

entry_points = \
{'console_scripts': ['bgmiget = bgmiget:main']}

setup_kwargs = {
    'name': 'bgmiget',
    'version': '0.1.3',
    'description': 'a way to download anime.',
    'long_description': '',
    'author': 'aoout',
    'author_email': 'wuz66280@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import requests
from bs4 import BeautifulSoup
from torrentp import TorrentDownloader

BASE_URL = "https://mikanani.me/"


class MiKanProject:
    def __init__(self) -> None:
        self.results = None

    def search(self, query):
        URL = BASE_URL + "Home/Search?searchstr=" + query
        page = requests.get(URL)
        soup = BeautifulSoup(page.content, "html.parser")

        self.results = [(i.find("a", class_="magnet-link-wrap").text, i.find(
            "a", class_="js-magnet magnet-link")["data-clipboard-text"]) for i in soup.find_all("tr", class_="js-search-results-row")]
        self.show_results()

    def download(self, path, index):
        if index != "all":
            url = self.results[index][1]
            torrent_file = TorrentDownloader(url, path)
            torrent_file.start_download()
        else:
            for i in range(len(self.results)):
                self.download(path, i)

    def show_results(self):
        for i, result in enumerate(self.results):
            print(str(i)+" -> "+result[0])
